###########################################################
###  Testing Linear Specification #########################
###########################################################

# load relevant packages
library(MASS)
library(expm)
library(orthopolynom)
library(psych)
library(EQL)
library(parallel)
library(SuppDists)

############################
### Set parameters   #######
############################

  n <- 500  #Sample Size
  N <- 1000 #Number of Monte Carlo Iterations
  
  eta  <- 0.7 # or 1 
  imag <- complex( imaginary = 1)
  ones<-rep(1,1,n)
  
  core.c <- 1    #Number of cores used for parallel computing
                  #If using windows then set core.c <- 1
  
  K <- 50    #Number of grid points to approximate the integral
  c.vec <- normOrder(K)  # Set vector of normal weights

####################################
### Set Tuning parameters   #######
####################################
  
  m_n <-  8 # or 12, 16
  
  k1 <- 2
  k2 <- 1  # or 2
  k_n<- (k1+1)+(k2+1)     # so k_n=5 or: k_n=6 when k2=2

##################################################
####### MONTE CARLO SIMULATION####################
################################################## 
Test.val.mat <- mat.or.vec(5,5)
      
MC.fct <- function(MC)
  {
  ######### Generate DATA   #############
  X <- rnorm(n,0,sqrt(2))
  
  
  B<- mvrnorm(n, c(0,0), matrix(c(1,.5,.5,1),2,2), empirical = FALSE, EISPACK = FALSE)
  B1 <- B[,1]
  B2 <- B[,2]
  B0 <- eta*rnorm(n)
  
  ############################
  ### Null Hypothesis  #######
  ############################
  
  Y <- B0 + B1*X
  
  ###Hermite Functions####
 BasX.mat<- mat.or.vec(n,m_n)
  for (i in (1:n)){
    BasX.mat[i,]<- pi^{-.25}*exp(-(X[i])^2/2)*hermite(X[i], 0:(m_n-1), FALSE)/sqrt(2^(0:(m_n-1))*factorial(0:(m_n-1)))
 }
  
  HX.mat <- crossprod(t(BasX.mat),tcrossprod(ginv(crossprod(BasX.mat)),BasX.mat))
  
  ## Estimate conditional characteristic function 
  phi.mat <- array(0,c(n,K))
  for(c.k in 1:K) {phi.mat[,c.k] <- crossprod(t(HX.mat),exp(imag*Y*c.vec[c.k])) }
  
 
  eval.bas <- function(x,l){sqrt(2*pi)*pi^{-.25}*exp(-x^2/2)*hermite(x, l, FALSE)/((-imag)^(l)*sqrt(2^(l)*factorial(l)))}
  
  Bas.mat<- mat.or.vec(n,k_n)
   Bas.f <- function(t){
     for (i in (1:n)){
       Bas.mat[i,] <- c(eval.bas(t,0:k1),eval.bas(t*X[i],0:k2))}  
     Bas.mat}

 Bas.f.ar <- array(NA, c(n, (k_n),K))
 for(i in 1:K){Bas.f.ar[,,i] <- Bas.f(c.vec[i])}
 
 #############################################
 #### Sieve Minimum Distance Estimation   ####
 #############################################
 
 ff0<-function(x){
   A<- phi.mat-do.call(cbind,lapply(seq_len(dim(Bas.f.ar)[3]),function(i) Bas.f.ar[,,i]%*%x))
   return(sum(abs(A^2)))}
 
  eval.bas <- function(x,i){exp(-x^2/2)*hermite(x, i, FALSE)/sqrt((pi)^{1/2}*2^i*factorial(i))}

 start.vec<-rep(0,k_n)
 dens.coef <- optim(start.vec, ff0)$par
      
  U.mat <- do.call(cbind,lapply(seq_len(dim(Bas.f.ar)[3]),function(i) Bas.f.ar[,,i]%*%dens.coef))
 
  crit.ar <- phi.mat -U.mat
  
  S0.n <- sum(abs(crit.ar)^2)/(K*n)    # Statistic S_n
  
  ###############################################################
  #### Estimation of mean and variance for standardization ######
  ###############################################################
  
  BasXo.mat <- tcrossprod(sqrtm(ginv(crossprod(BasX.mat))),BasX.mat)
  
  
  for(c.k in 1:K) {crit.ar[,c.k] <- exp(imag*Y*c.vec[c.k])-U.mat[,c.k] }
  
  Bas2.f.ar <- array(NA, c(n, k_n,K))
  for(i in 1:K){Bas2.f.ar[,,i] <- Bas.f(c.vec[i])}
  U2.mat <- do.call(cbind,lapply(seq_len(dim(Bas2.f.ar)[3]),function(i) Bas2.f.ar[,,i]%*%dens.coef))
  
  crit2.ar<- crit.ar
  for(c.k in 1:K) {crit2.ar[,c.k] <- Conj(exp(imag*Y*c.vec[c.k])-U2.mat[,c.k]) }
  
  mu.vec<- vector()
  for(c.k in 1:K) {mu.vec[c.k] <- tr(BasXo.mat%*%diag(as.vector(abs(crit.ar[,c.k] )^2))%*%t(BasXo.mat))}
  mu <- sum(mu.vec)/K
  
  eps.vec <- vector()
  for(c.k in 1:K) { eps.vec[c.k] <- sum((abs(crossprod(BasXo.mat*crit.ar[,c.k],BasXo.mat*crit2.ar[,c.k])))^2) }
  
  
  ###### Now the test statistic is given by 
  test0 <- Re((n*S0.n - mu)/(sqrt(2)*sqrt(sum(eps.vec)/K)))


############################
### 1st alternative  #######
############################
  c<- var(B0 +B1*X)/var(B0 +B1*X+B2*X^2)
  Y <- (B0 +B1*X+B2*X^2)*sqrt(c)
  
  
  ###Construct Test Statistic####
  
  phi.mat <- array(0,c(n,K))
  for(c.k in 1:K) {phi.mat[,c.k] <- crossprod(t(HX.mat),(exp(imag*Y*c.vec[c.k]))) }
  
  eval.bas <- function(x,l){sqrt(2*pi)*pi^{-.25}*exp(-x^2/2)*hermite(x, l, FALSE)/((-imag)^(l)*sqrt(2^(l)*factorial(l)))}
  
  Bas.mat<- mat.or.vec(n,k_n)
  Bas.f <- function(t){
    for (i in (1:n)){
      Bas.mat[i,] <- c(eval.bas(t,0:k1),eval.bas(t*X[i],0:k2))}  
    Bas.mat}
  
  Bas.f.ar <- array(NA, c(n, (k_n),K))
  for(i in 1:K){Bas.f.ar[,,i] <- Bas.f(c.vec[i])}

#############################################
#### Sieve Minimum Distance Estimation   ####
#############################################
  
  ff0<-function(x){
    A<- phi.mat-do.call(cbind,lapply(seq_len(dim(Bas.f.ar)[3]),function(i) Bas.f.ar[,,i]%*%x))
    return(sum(abs(A^2)))}
  
  eval.bas <- function(x,i){exp(-x^2/2)*hermite(x, i, FALSE)/sqrt((pi)^{1/2}*2^i*factorial(i))}
  
  dens.coef <- optim(start.vec, ff0)$par
  
  U.mat <- do.call(cbind,lapply(seq_len(dim(Bas.f.ar)[3]),function(i) Bas.f.ar[,,i]%*%dens.coef))
  
  crit.ar <- phi.mat -U.mat
  
  S0.n <- sum(abs(crit.ar)^2)/(K*n)   # Statistic S_n

###############################################################
#### Estimation of mean and variance for standardization ######
###############################################################
  
  for(c.k in 1:K) {crit.ar[,c.k] <- exp(imag*Y*c.vec[c.k])-U.mat[,c.k] }
  
  Bas2.f.ar <- array(NA, c(n, k_n,K))
  for(i in 1:K){Bas2.f.ar[,,i] <- Bas.f(c.vec[i])}
  U2.mat <- do.call(cbind,lapply(seq_len(dim(Bas2.f.ar)[3]),function(i) Bas2.f.ar[,,i]%*%dens.coef))
  
  crit2.ar<- crit.ar
  for(c.k in 1:K) {crit2.ar[,c.k] <- Conj(exp(imag*Y*c.vec[c.k])-U2.mat[,c.k]) }
  
  mu.vec<- vector()
  for(c.k in 1:K) {mu.vec[c.k] <- tr(BasXo.mat%*%diag(as.vector(abs(crit.ar[,c.k] )^2))%*%t(BasXo.mat))}
  mu <- sum(mu.vec)/K
  
  eps.vec <- vector()
  for(c.k in 1:K) { eps.vec[c.k] <- sum((abs(t(BasXo.mat*crit.ar[,c.k])%*%(BasXo.mat*crit2.ar[,c.k])))^2) }
  

  
  ###### Now the test statistic is given by 
  test1 <- Re((n*S0.n - mu)/(sqrt(2)*sqrt(sum(eps.vec)/K)))
  

############################
### 2nd alternative  #######
############################
  c<- var(B0 +B1*X)/var(B0 +B1*X+B2*sqrt(abs(X)))
  Y <- (B0 +B1*X+B2*sqrt(abs(X)))*sqrt(c)
   
  phi.mat <- array(0,c(n,K))
  for(c.k in 1:K) {phi.mat[,c.k] <- crossprod(t(HX.mat),(exp(imag*Y*c.vec[c.k]))) }
  
  eval.bas <- function(x,l){sqrt(2*pi)*pi^{-.25}*exp(-x^2/2)*hermite(x, l, FALSE)/((-imag)^(l)*sqrt(2^(l)*factorial(l)))}
  
  Bas.mat<- mat.or.vec(n,k_n)
  Bas.f <- function(t){
    for (i in (1:n)){
      Bas.mat[i,] <- c(eval.bas(t,0:k1),eval.bas(t*X[i],0:k2))}  
    Bas.mat}
  
  Bas.f.ar <- array(NA, c(n, (k_n),K))
  for(i in 1:K){Bas.f.ar[,,i] <- Bas.f(c.vec[i])}

#############################################
#### Sieve Minimum Distance Estimation   ####
#############################################
  ff0<-function(x){
    A<- phi.mat-do.call(cbind,lapply(seq_len(dim(Bas.f.ar)[3]),function(i) Bas.f.ar[,,i]%*%x))
    return(sum(abs(A^2)))}
  
  eval.bas <- function(x,i){exp(-x^2/2)*hermite(x, i, FALSE)/sqrt((pi)^{1/2}*2^i*factorial(i))}
  
  dens.coef <- optim(start.vec, ff0)$par
  
  U.mat <- do.call(cbind,lapply(seq_len(dim(Bas.f.ar)[3]),function(i) Bas.f.ar[,,i]%*%dens.coef))
  
  crit.ar <- phi.mat -U.mat
  
  S0.n <- sum(abs(crit.ar)^2)/(K*n)   # Statistic S_n

###############################################################
#### Estimation of mean and variance for standardization ######
###############################################################
  
  for(c.k in 1:K) {crit.ar[,c.k] <- exp(imag*Y*c.vec[c.k])-U.mat[,c.k] }
  
  Bas2.f.ar <- array(NA, c(n, k_n,K))
  for(i in 1:K){Bas2.f.ar[,,i] <- Bas.f(c.vec[i])}
  U2.mat <- do.call(cbind,lapply(seq_len(dim(Bas2.f.ar)[3]),function(i) Bas2.f.ar[,,i]%*%dens.coef))
  
  crit2.ar<- crit.ar
  for(c.k in 1:K) {crit2.ar[,c.k] <- Conj(exp(imag*Y*c.vec[c.k])-U2.mat[,c.k]) }
  
  mu.vec<- vector()
  for(c.k in 1:K) {mu.vec[c.k] <- tr(BasXo.mat%*%diag(as.vector(abs(crit.ar[,c.k] )^2))%*%t(BasXo.mat))}
  mu <- sum(mu.vec)/K
  
  eps.vec <- vector()
  for(c.k in 1:K) { eps.vec[c.k] <- sum((abs(crossprod(BasXo.mat*crit.ar[,c.k],BasXo.mat*crit2.ar[,c.k])))^2) }
  
  
  ###### Now the test statistic is given by 
  test2 <- Re((n*S0.n - mu)/(sqrt(2)*sqrt(sum(eps.vec)/K)))
  

c(test0, test1, test2)

}


test <- t(mcmapply(MC.fct,1:N, mc.cores = core.c))


#### Compute Empirical Rejection Probabilities at nominal level 0.05
Test.Matrix <- matrix(ncol=3,nrow=N)
for (l in 1:3){  for (MC in 1:N){ Test.Matrix[MC,l]=as.integer(test[MC,l]>=qnorm(0.95))}}

print(colMeans(Test.Matrix))
